@@ -4,7 +4,9 @@
 import contextlib
 import errno
 import glob
+import gzip
 import itertools
+import io
 import logging
 import multiprocessing
 import operator
@@ -16,7 +18,12 @@ import signal
 from typing import Any, Dict, Union, Optional, List, TextIO, Mapping
 
 from ._base import Command, CommandExit, InvalidArgument
-from .._utils import guess_sequences_format, in_context, patch_showwarnings
+from .._utils import (
+    guess_sequences_format,
+    in_context,
+    patch_showwarnings,
+    ProgressReader,
+)
 
 
 class Annotate(Command):  # noqa: D101
@@ -50,7 +57,9 @@ class Annotate(Command):  # noqa: D101
 
         Parameters - Domain Annotation:
             -e <e>, --e-filter <e>        the e-value cutoff for protein domains
-                                          to be included. [default: 1e-5]
+                                          to be included.
+            -p <p>, --p-filter <p>        the p-value cutoff for protein domains
+                                          to be included. [default: 1e-9]
 
         Parameters - Debug:
             --hmm <hmm>                   the path to one or more alternative
@@ -60,7 +69,20 @@ class Annotate(Command):  # noqa: D101
     def _check(self) -> typing.Optional[int]:
         super()._check()
         try:
-            self.e_filter = self._check_flag("--e-filter", float, lambda x: 0 <= x <= 1, hint="real number between 0 and 1")
+            self.e_filter = self._check_flag(
+                "--e-filter",
+                float,
+                lambda x: x > 0,
+                hint="real number above 0",
+                optional=True,
+            )
+            self.p_filter = self._check_flag(
+                "--p-filter",
+                float,
+                lambda x: x > 0,
+                hint="real number above 0",
+                optional=True,
+            )
             self.jobs = self._check_flag("--jobs", int, lambda x: x >= 0, hint="positive or null integer")
             self.format = self._check_flag("--format")
             self.genome = self._check_flag("--genome")
@@ -75,11 +97,16 @@ class Annotate(Command):  # noqa: D101
 
         for path in self.hmm:
             base = os.path.basename(path)
+            file = open(path, "rb")
             if base.endswith(".gz"):
                 base, _ = os.path.splitext(base)
+                file = gzip.GzipFile(fileobj=file)
             base, _ = os.path.splitext(base)
-            with pyhmmer.plan7.HMMFile(path) as hmm_file:
-                size = sum(1 for _ in hmm_file)
+            self.info("Counting", "profiles in HMM file", repr(path), level=1)
+            with file:
+                with pyhmmer.plan7.HMMFile(file) as hmm_file:
+                    size = sum(1 for _ in hmm_file)
+            self.success("Found", size, "profiles in HMM file", repr(path), level=1)
             yield HMM(
                 id=base,
                 version="?",
@@ -94,6 +121,7 @@ class Annotate(Command):  # noqa: D101
     def _load_sequences(self):
         from Bio import SeqIO
 
+        # guess format or use the one given in CLI
         if self.format is not None:
             format = self.format
             self.info("Using", "user-provided sequence format", repr(format), level=2)
@@ -102,13 +130,20 @@ class Annotate(Command):  # noqa: D101
             format = guess_sequences_format(self.genome)
             self.success("Detected", "format of input as", repr(format), level=2)
 
+        # get filesize and unit
+        input_size = os.stat(self.genome).st_size
+        total, scale, unit = ProgressReader.scale_size(input_size)
+        task = self.progress.add_task("Loading sequences", total=total, unit=unit, precision=".1f")
+
+        # load sequences
         self.info("Loading", "sequences from genomic file", repr(self.genome), level=1)
         try:
-            sequences = list(SeqIO.parse(self.genome, format))
+            with ProgressReader(open(self.genome, "rb"), self.progress, task, scale) as f:
+                sequences = list(SeqIO.parse(io.TextIOWrapper(f), format))
         except FileNotFoundError as err:
             self.error("Could not find input file:", repr(self.genome))
             raise CommandExit(e.errno) from err
-        except Exception as err:
+        except ValueError as err:
             self.error("Failed to load sequences:", err)
             raise CommandExit(getattr(err, "errno", 1)) from err
         else:
@@ -122,7 +157,7 @@ class Annotate(Command):  # noqa: D101
         orf_finder = PyrodigalFinder(metagenome=True, cpus=self.jobs)
 
         unit = "contigs" if len(sequences) > 1 else "contig"
-        task = self.progress.add_task(description="ORFs finding", total=len(sequences), unit=unit)
+        task = self.progress.add_task(description="ORFs finding", total=len(sequences), unit=unit, precision="")
 
         def callback(record, found, total):
             self.success("Found", found, "genes in record", repr(record.id), level=2)
@@ -137,9 +172,9 @@ class Annotate(Command):  # noqa: D101
 
         # Run all HMMs over ORFs to annotate with protein domains
         hmms = list(self._custom_hmms() if self.hmm else embedded_hmms())
-        task = self.progress.add_task(description=f"HMM annotation", unit="HMMs", total=len(hmms))
+        task = self.progress.add_task(description=f"HMM annotation", unit="HMMs", total=len(hmms), precision="")
         for hmm in self.progress.track(hmms, task_id=task, total=len(hmms)):
-            task = self.progress.add_task(description=f"{hmm.id} v{hmm.version}", total=hmm.size, unit="domains")
+            task = self.progress.add_task(description=f"{hmm.id} v{hmm.version}", total=hmm.size, unit="domains", precision="")
             callback = lambda h, t: self.progress.update(task, advance=1)
             self.info("Starting", f"annotation with [bold blue]{hmm.id} v{hmm.version}[/]", level=2)
             features = PyHMMER(hmm, self.jobs).run(genes, progress=callback)
@@ -150,14 +185,8 @@ class Annotate(Command):  # noqa: D101
         count = sum(1 for gene in genes for domain in gene.protein.domains)
         self.success("Found", count, "domains across all proteins", level=1)
 
-        # Filter i-evalue
-        self.info("Filtering", "results with e-value under", self.e_filter, level=1)
-        key = lambda d: d.i_evalue < self.e_filter
-        for gene in genes:
-            gene.protein.domains = list(filter(key, gene.protein.domains))
-
-        count = sum(1 for gene in genes for domain in gene.protein.domains)
-        self.info("Using", "remaining", count, "domains", level=2)
+        # Filter i-evalue and p-value if required
+        genes = self._filter_domains(genes)
 
         # Sort genes
         self.info("Sorting", "genes by coordinates", level=2)
@@ -167,6 +196,23 @@ class Annotate(Command):  # noqa: D101
 
         return genes
 
+    def _filter_domains(self, genes):
+        # Filter i-evalue and p-value if required
+        if self.e_filter is not None:
+            self.info("Filtering", "domains with e-value under", self.e_filter, level=1)
+            key = lambda d: d.i_evalue < self.e_filter
+            for gene in genes:
+                gene.protein.domains = list(filter(key, gene.protein.domains))
+        if self.p_filter is not None:
+            self.info("Filtering", "domains with p-value under", self.p_filter, level=1)
+            key = lambda d: d.pvalue < self.p_filter
+            for gene in genes:
+                gene.protein.domains = list(filter(key, gene.protein.domains))
+        if self.p_filter is not None or self.e_filter is not None:
+            count = sum(1 for gene in genes for domain in gene.protein.domains)
+            self.info("Using", "remaining", count, "domains", level=1)
+        return genes
+
     def _write_feature_table(self, genes):
         from ...model import FeatureTable
 
