@@ -9,14 +9,19 @@ import os
 import operator
 import multiprocessing
 import random
+import signal
 import typing
 from typing import Any, Dict, Union, Optional, List, TextIO, Mapping
 
-from ._base import Command, CommandExit, InvalidArgument
+import docopt
+
 from .._utils import patch_showwarnings
+from ._base import Command, CommandExit, InvalidArgument
+from .annotate import Annotate
+from .train import Train
 
 
-class Cv(Command):  # noqa: D101
+class Cv(Train):  # noqa: D101
 
     summary = "perform cross validation on a training set."
 
@@ -26,8 +31,8 @@ class Cv(Command):  # noqa: D101
         gecco cv  - {cls.summary}
 
         Usage:
-            gecco cv kfold -f <table> [-c <data>] [options]
-            gecco cv loto  -f <table>  -c <data>  [options]
+            gecco cv kfold --features <table>... --clusters <table> [options]
+            gecco cv loto  --features <table>... --clusters <table> [options]
 
         Arguments:
             -f <data>, --features <table>   a domain annotation table, used to
@@ -46,7 +51,16 @@ class Cv(Command):  # noqa: D101
 
         Parameters - Domain Annotation:
             -e <e>, --e-filter <e>          the e-value cutoff for domains to
-                                            be included [default: 1e-5]
+                                            be included.
+            -p <p>, --p-filter <p>          the p-value cutoff for domains
+                                            to be included. [default: 1e-9]
+
+        Parameters - Training Data:
+            --no-shuffle                    disable shuffling of the data
+                                            before fitting the model.
+            --seed <N>                      the seed to initialize the RNG
+                                            with for shuffling operations.
+                                            [default: 42]
 
         Parameters - Training:
             --c1 <C1>                       parameter for L1 regularisation.
@@ -58,90 +72,42 @@ class Cv(Command):  # noqa: D101
                                             [default: group]
             --overlap <N>                   how much overlap to consider if
                                             features overlap. [default: 2]
-            --splits <N>                    number of folds for cross-validation
-                                            (if running `kfold`). [default: 10]
             --select <N>                    fraction of most significant features
                                             to select from the training data.
-            --shuffle                       enable shuffling of stratified rows.
+            --correction <method>           the multiple test correction method
+                                            to use when computing significance
+                                            with multiple Fisher tests.
+
+        Parameters - Cross-validation:
+            --splits <N>                    number of folds for cross-validation
+                                            (if running `kfold`). [default: 10]
 
         """
 
     def _check(self) -> typing.Optional[int]:
+        if not isinstance(self.args, docopt.DocoptExit):
+            self.args["--output-dir"] = "."
         super()._check()
         try:
-            self.feature_type = self._check_flag(
-                "--feature-type",
-                str,
-                lambda x: x in {"single", "overlap", "group"},
-                hint="'single', 'overlap' or 'group'"
-            )
-            self.overlap = self._check_flag(
-                "--overlap",
+            self.output = self._check_flag("--output", str)
+            self.splits = self._check_flag(
+                "--splits",
                 int,
                 lambda x: x > 0,
-                hint="positive integer",
+                hint="positive integer"
             )
-            self.c1 = self._check_flag("--c1", float, hint="real number")
-            self.c2 = self._check_flag("--c2", float, hint="real number")
-            self.splits = self._check_flag("--splits", int, lambda x: x>1, hint="integer greater than 1")
-            self.e_filter = self._check_flag(
-                "--e-filter",
-                float,
-                lambda x: 0 <= x <= 1,
-                hint="real number between 0 and 1"
-            )
-            self.select = self._check_flag(
-                "--select",
-                lambda x: x if x is None else float(x),
-                lambda x: x is None or 0 <= x <= 1,
-                hint="real number between 0 and 1"
-            )
-            self.jobs = self._check_flag(
-                "--jobs",
-                int,
-                lambda x: x >= 0,
-                hint="positive or null integer"
-            )
-            self.features = self._check_flag("--features")
-            self.clusters = self._check_flag("--clusters")
             self.loto = self.args["loto"]
-            self.output = self.args["--output"]
         except InvalidArgument:
             raise CommandExit(1)
 
     # --
 
-    def _load_features(self):
-        from ...model import FeatureTable
-
-        self.info("Loading", "features table from file", repr(self.features))
-        with open(self.features) as in_:
-            return FeatureTable.load(in_)
-
-    def _convert_to_genes(self, features):
-        self.info("Converting", "features to genes")
-        gene_count = len(set(features.protein_id))
-        unit = "gene" if gene_count == 1 else "genes"
-        task = self.progress.add_task("Feature conversion", total=gene_count, unit=unit)
-        genes = list(self.progress.track(
-            features.to_genes(),
-            total=gene_count,
-            task_id=task
-        ))
-
-        self.info("Sorting", "genes by genomic coordinates")
-        genes.sort(key=operator.attrgetter("source.id", "start", "end"))
-        self.info("Sorting", "domains by protein coordinates")
-        for gene in genes:
-            gene.protein.domains.sort(key=operator.attrgetter("start", "end"))
-        return genes
-
     def _group_genes(self, genes):
         self.info("Grouping", "genes by source sequence")
         groups = itertools.groupby(genes, key=operator.attrgetter("source.id"))
         seqs = [sorted(group, key=operator.attrgetter("start")) for _, group in groups]
-        if self.args["--shuffle"]:
-            self.info("Shuffling training data sequences")
+        if not self.no_shuffle:
+            self.info("Shuffling", "training data sequences")
             random.shuffle(seqs)
         return seqs
 
@@ -172,11 +138,13 @@ class Cv(Command):  # noqa: D101
         import sklearn.model_selection
         return list(sklearn.model_selection.KFold(self.splits).split(seqs))
 
-    def _get_train_data(self, train_indices, seqs):
+    @staticmethod
+    def _get_train_data(train_indices, seqs):
         # extract train data
         return [gene for i in train_indices for gene in seqs[i]]
 
-    def _get_test_data(self, test_indices, seqs):
+    @staticmethod
+    def _get_test_data(test_indices, seqs):
         # make a clean copy of the test data without gene probabilities
         test_data = [copy.deepcopy(gene) for i in test_indices for gene in seqs[i]]
         for gene in test_data:
@@ -186,15 +154,7 @@ class Cv(Command):  # noqa: D101
     def _fit_predict(self, train_data, test_data):
         from ...crf import ClusterCRF
 
-        # fit and predict the CRF for the current fold
-        crf = ClusterCRF(
-            self.feature_type,
-            algorithm="lbfgs",
-            overlap=self.overlap,
-            c1=self.c1,
-            c2=self.c2,
-        )
-        crf.fit(train_data, cpus=self.jobs, select=self.select)
+        crf = self._fit_model(train_data)
         return crf.predict_probabilities(test_data, cpus=self.jobs)
 
     def _write_fold(self, fold, genes, append=False):
@@ -212,11 +172,15 @@ class Cv(Command):  # noqa: D101
             self._check()
             ctx.enter_context(self.progress)
             ctx.enter_context(patch_showwarnings(self._showwarnings))
+            # seed RNG
+            self._seed_rng()
             # load features
             features = self._load_features()
-            self.success("Loaded", len(features), "feature annotations")
             genes = self._convert_to_genes(features)
-            self.success("Recoverd", len(genes), "genes from the feature annotations")
+            del features
+            # load clusters and label genes inside clusters
+            clusters = self._load_clusters()
+            genes = self._label_genes(genes, clusters)
             seqs = self._group_genes(genes)
             self.success("Grouped", "genes into", len(seqs), "sequences")
             # split CV folds
@@ -226,7 +190,7 @@ class Cv(Command):  # noqa: D101
                 splits = self._kfold_splits(seqs)
             # run CV
             unit = "fold" if len(splits) == 1 else "folds"
-            task = self.progress.add_task(description="Cross-Validation", total=len(splits), unit=unit)
+            task = self.progress.add_task(description="Cross-Validation", total=len(splits), unit=unit, precision="")
             self.info("Performing cross-validation")
             for i, (train_indices, test_indices) in enumerate(self.progress.track(splits, task_id=task)):
                 train_data = self._get_train_data(train_indices, seqs)
