@@ -61,7 +61,9 @@ class Run(Annotate):  # noqa: D101
 
         Parameters - Domain Annotation:
             -e <e>, --e-filter <e>        the e-value cutoff for protein domains
-                                          to be included. [default: 1e-5]
+                                          to be included.
+            -p <p>, --p-filter <p>        the p-value cutoff for protein domains
+                                          to be included. [default: 1e-9]
 
         Parameters - Cluster Detection:
             -c, --cds <N>                 the minimum number of coding sequences a
@@ -84,7 +86,20 @@ class Run(Annotate):  # noqa: D101
         Command._check(self)
         try:
             self.cds = self._check_flag("--cds", int, lambda x: x > 0, hint="positive integer")
-            self.e_filter = self._check_flag("--e-filter", float, lambda x: 0 <= x <= 1, hint="real number between 0 and 1")
+            self.e_filter = self._check_flag(
+                "--e-filter",
+                float,
+                lambda x: x > 0,
+                hint="real number above 0",
+                optional=True,
+            )
+            self.p_filter = self._check_flag(
+                "--p-filter",
+                float,
+                lambda x: x > 0,
+                hint="real number above 0",
+                optional=True,
+            )
             if self.args["--threshold"] is None:
                 self.threshold = 0.4 if self.args["--postproc"] == "gecco" else 0.6
             else:
@@ -109,7 +124,7 @@ class Run(Annotate):  # noqa: D101
             os.makedirs(self.output_dir, exist_ok=True)
         except OSError as err:
             self.error("Could not create output directory: {}", err)
-            raise CommandExit(e.errno) from err
+            raise CommandExit(err.errno) from err
 
         # Check if output files already exist
         base, _ = os.path.splitext(os.path.basename(self.genome))
@@ -132,7 +147,7 @@ class Run(Annotate):  # noqa: D101
 
         self.info("Predicting", "cluster probabilitites with the CRF model", level=1)
         unit = "genes" if len(genes) > 1 else "gene"
-        task = self.progress.add_task("Prediction", total=len(genes), unit=unit)
+        task = self.progress.add_task("Prediction", total=len(genes), unit=unit, precision="")
         return list(crf.predict_probabilities(
             self.progress.track(genes, task_id=task, total=len(genes)),
             cpus=self.jobs
@@ -155,7 +170,7 @@ class Run(Annotate):  # noqa: D101
 
         total = len({gene.source.id for gene in genes})
         unit = "contigs" if total > 1 else "contig"
-        task = self.progress.add_task("Segmentation", total=total, unit=unit)
+        task = self.progress.add_task("Segmentation", total=total, unit=unit, precision="")
 
         clusters = []
         gene_groups = itertools.groupby(genes, lambda g: g.source.id)
@@ -171,7 +186,7 @@ class Run(Annotate):  # noqa: D101
         self.info("Predicting", "BGC types", level=1)
 
         unit = "cluster" if len(clusters) == 1 else "clusters"
-        task = self.progress.add_task("Type prediction", total=len(clusters), unit=unit)
+        task = self.progress.add_task("Type prediction", total=len(clusters), unit=unit, precision="")
 
         clusters_new = []
         classifier = TypeClassifier.trained(self.model)
@@ -292,7 +307,8 @@ class Run(Annotate):  # noqa: D101
             self._write_clusters(clusters)
             if self.antismash_sideload:
                 self._write_sideload_json(clusters)
-            self.success("Found", len(clusters), "biosynthetic gene clusters", level=0)
+            unit = "cluster" if len(clusters) == 1 else "clusters"
+            self.success("Found", len(clusters), "biosynthetic gene", unit, level=0)
         except CommandExit as cexit:
             return cexit.code
         except KeyboardInterrupt:
