@@ -72,8 +72,8 @@ class ClusterCRF(object):
         """
         # get the path to the pickled model and read its signature file
         if model_path is not None:
-            pkl_file = os.path.join(model_path, "model.pkl")
-            md5_file = os.path.join(model_path, "model.pkl.md5")
+            pkl_file = open(os.path.join(model_path, "model.pkl"), "rb")
+            md5_file = open(os.path.join(model_path, "model.pkl.md5"))
         else:
             pkl_file = importlib_resources.open_binary(__name__, "model.pkl")
             md5_file = importlib_resources.open_text(__name__, "model.pkl.md5")
@@ -131,6 +131,7 @@ class ClusterCRF(object):
         self.overlap: int = overlap
         self.algorithm = algorithm
         self.pool_factory = pool_factory
+        self.significance: Optional[Dict[str, float]] = None
         self.significant_features: Optional[FrozenSet[str]] = None
         self.model = sklearn_crfsuite.CRF(
             algorithm=algorithm,
@@ -170,7 +171,15 @@ class ClusterCRF(object):
         # probabilities set
         return list(itertools.chain.from_iterable(annotated_seqs)) # type: ignore
 
-    def fit(self, genes: Iterable[Gene], select: Optional[float] = None, shuffle: bool = True, *, cpus: Optional[int] = None) -> None:
+    def fit(
+        self,
+        genes: Iterable[Gene],
+        select: Optional[float] = None,
+        shuffle: bool = True,
+        *,
+        cpus: Optional[int] = None,
+        correction_method: Optional[str] = None,
+    ) -> None:
         _cpus = os.cpu_count() if not cpus else cpus
         # select the feature extraction method
         if self.feature_type == "group":
@@ -197,9 +206,19 @@ class ClusterCRF(object):
             if select <= 0 or select > 1:
                 raise ValueError(f"invalid value for select: {select}")
             # find most significant features
-            sig = fisher_significance(g.protein for seq in seqs for g in seq)
+            self.significance = sig = fisher_significance(
+                (g.protein for seq in seqs for g in seq),
+                correction_method=correction_method,
+            )
             sorted_sig = sorted(sig, key=sig.get)[:int(select*len(sig))]
             self.significant_features = frozenset(sorted_sig)
+            # check that we don't select "random" domains
+            if sig[sorted_sig[-1]] == 1.0:
+                warnings.warn(
+                    "Selected features still include domains with a p-value "
+                    "of 1, consider reducing the selected fraction.",
+                    UserWarning
+                )
             # remove non significant domains
             for i, seq in enumerate(seqs):
                 for j, gene in enumerate(seq):
@@ -215,5 +234,11 @@ class ClusterCRF(object):
             X = pool.map(extract_features, seqs)
             Y = pool.map(extract_labels, seqs)
 
+        # check labels
+        if all(y == "1" for y in Y):
+            raise ValueError("only positives labels found, something is wrong.")
+        elif all(y == "0" for y in Y):
+            raise ValueError("only negative labels found, something is wrong.")
+
         # fit the model
         self.model.fit(X, Y)
