@@ -7,6 +7,7 @@ import datetime
 import enum
 import functools
 import itertools
+import math
 import operator
 import re
 import typing
@@ -105,6 +106,8 @@ class Domain:
             (e.g. ``Pfam``, ``Panther``).
         i_evalue (`float`): The independent e-value reported by ``hmmsearch``
             that measures how reliable the domain annotation is.
+        pvalue (`float`): The p-value reported by ``hmmsearch`` that measure
+            how likely the domain score is.
         probability (`float`, optional): The probability that this domain
             is part of a BGC, or `None` if no prediction has been made yet.
         qualifiers (`dict`, optional): A dictionary of feature qualifiers that
@@ -118,6 +121,7 @@ class Domain:
     end: int
     hmm: str
     i_evalue: float
+    pvalue: float
     probability: Optional[float] = None
     qualifiers: Mapping[str, Union[str, List[str]]] = field(default_factory=dict)
 
@@ -125,7 +129,7 @@ class Domain:
         """Copy the current domain and assign it a BGC probability.
         """
         return Domain(
-            self.name, self.start, self.end, self.hmm, self.i_evalue,
+            self.name, self.start, self.end, self.hmm, self.i_evalue, self.pvalue,
             probability, self.qualifiers
         )
 
@@ -244,7 +248,7 @@ class Cluster:
         id (`str`): The identifier of the gene cluster.
         genes (`list` of `~gecco.model.Gene`): A list of the genes belonging
             to this gene cluster.
-        types (`list` of `str`): The list of the putative types of product
+        types (`gecco.model.ProductType`): The putative types of product
             synthesized by this gene cluster, according to similarity in
             domain composition with curated clusters.
         types_probabilities (`list` of `float`): The probability with which
@@ -313,6 +317,8 @@ class Cluster:
         self,
         all_possible: Optional[Sequence[str]] = None,
         normalize: bool = True,
+        minlog_weights: bool = False,
+        pvalue: bool = True,
     ) -> numpy.ndarray:
         """Compute weighted domain composition with respect to ``all_possible``.
 
@@ -321,6 +327,14 @@ class Cluster:
                 all domain names to consider when computing domain composition
                 for the BGC. If `None` given, then only domains within the
                 cluster are taken into account.
+            normalize (`bool`): Normalize the composition vector so that it
+                sums to 1.
+            minlog_weights (`bool`): Compute weight for each domain as
+                :math:`-log_10(v)` (where :math:`v` is either the ``pvalue``
+                or the ``i_evalue``, depending on the value of ``normalize``).
+                Use :math:`1 - v` otherwise.
+            pvalue (`bool`): Compute composition weights using the ``pvalue``
+                of each domain, instead of the ``i_evalue``.
 
         Returns:
             `~numpy.ndarray`: A numerical array containing the relative domain
@@ -329,7 +343,12 @@ class Cluster:
         """
         domains = [d for gene in self.genes for d in gene.protein.domains]
         names = numpy.array([domain.name for domain in domains])
-        weights = numpy.array([1 - domain.i_evalue for domain in domains])
+        field = operator.attrgetter("pvalue" if pvalue else "i_evalue")
+        if minlog_weights:
+            weights = numpy.array([- math.log10(field(domain)) for domain in domains])
+        else:
+            weights = numpy.array([1 - field(domain) for domain in domains])
+
         unique_names = set(names)
         if all_possible is None:
             all_possible = numpy.unique(names)
@@ -394,7 +413,7 @@ class Cluster:
         structured_comment['GECCO-Data'] = {
             "version": f"GECCO v{__version__}",
             "creation_date": now.isoformat(),
-            "biosyn_class": ",".join(ty.name for ty in self.type.unpack()),
+            "biosyn_class": ";".join(sorted(ty.name for ty in self.type.unpack())),
             "alkaloid_probability": self.type_probabilities.get(ProductType.Alkaloid, 0.0),
             "polyketide_probability": self.type_probabilities.get(ProductType.Polyketide, 0.0),
             "ripp_probability": self.type_probabilities.get(ProductType.RiPP, 0.0),
@@ -456,6 +475,7 @@ class FeatureTable(Dumpable, Sized):
     domain: List[str] = field(default_factory = list)
     hmm: List[str] = field(default_factory = list)
     i_evalue: List[float] = field(default_factory = lambda: array("d"))     # type: ignore
+    pvalue: List[float] = field(default_factory = lambda: array("d"))       # type: ignore
     domain_start: List[int] = field(default_factory = lambda: array("l"))   # type: ignore
     domain_end: List[int] = field(default_factory = lambda: array("l"))     # type: ignore
     bgc_probability: List[Optional[float]] = field(default_factory = list)
@@ -472,6 +492,7 @@ class FeatureTable(Dumpable, Sized):
         domain: str
         hmm: str
         i_evalue: float
+        pvalue: float
         domain_start: int
         domain_end: int
         bgc_probability: Optional[float]
@@ -491,6 +512,7 @@ class FeatureTable(Dumpable, Sized):
                 table.domain.append(domain.name)
                 table.hmm.append(domain.hmm)
                 table.i_evalue.append(domain.i_evalue)
+                table.pvalue.append(domain.pvalue)
                 table.domain_start.append(domain.start)
                 table.domain_end.append(domain.end)
                 table.bgc_probability.append(domain.probability)
@@ -519,7 +541,7 @@ class FeatureTable(Dumpable, Sized):
             protein = Protein(rows[0].protein_id, seq=None)
             gene = Gene(source, rows[0].start, rows[0].end, strand, protein)
             for row in rows:
-                domain = Domain(row.domain, row.domain_start, row.domain_end, row.hmm, row.i_evalue, row.bgc_probability)
+                domain = Domain(row.domain, row.domain_start, row.domain_end, row.hmm, row.i_evalue, row.pvalue, row.bgc_probability)
                 gene.protein.domains.append(domain)
             yield gene
 
@@ -536,6 +558,13 @@ class FeatureTable(Dumpable, Sized):
             frame[column] = getattr(self, column)
         return frame
 
+    def __iadd__(self, rhs: "FeatureTable") -> "FeatureTable":
+        if not isinstance(rhs, FeatureTable):
+            return NotImplemented
+        for col in self.__annotations__:
+            getattr(self, col).extend(getattr(rhs, col))
+        return self
+
     def __bool__(self) -> bool:  # noqa: D105
         return len(self) != 0
 
@@ -578,6 +607,11 @@ class FeatureTable(Dumpable, Sized):
         """
         writer = csv.writer(fh, dialect=dialect)
         columns = list(self.__annotations__)
+
+        # do not write optional columns if they are completely empty
+        if all(proba is None for proba in self.bgc_probability):
+            columns.remove("bgc_probability")
+
         if header:
             writer.writerow(columns)
         for row in self:
@@ -591,31 +625,27 @@ class FeatureTable(Dumpable, Sized):
         reader = csv.reader(fh, dialect=dialect)
         header = next(reader)
 
-        # find the columns and the type of each column based on the annotations
-        columns = [
-            (
-                (header.index(col) if col in header else None),
-                col,
-                getattr(table, col).append,
-                getattr(ty.__args__[0], "__args__", ty.__args__)[0], # concrete type
-            )
-            for col, ty in cls.__annotations__.items()
-        ]
+        # get the name of each column
+        columns = {i:col for i, col in enumerate(header)}
 
-        # check that if a column is missing, it is Optional
-        missing = [
-            c
-            for i, c, _, _ in columns
-            if i is None and not isinstance(cls.__annotations__[c].__args__[0], type)
-        ]
-        if missing:
-            raise ValueError("table is missing columns: {}".format(", ".join(missing)))
+        # check that if a column is missing, it is one of the optional values
+        missing = set(cls.__annotations__).difference(columns.values())
+        missing_required = missing.difference({"bgc_probability"})
+        if missing_required:
+            raise ValueError("table is missing columns: {}".format(", ".join(missing_required)))
 
         # extract elements from the CSV rows
         for row in reader:
-            for index, _, append, ty in columns:
-                append(None if index is None else ty(row[index]))
-
+            for col in missing:
+                getattr(table, col).append(None)
+            for i,value in enumerate(row):
+                col = columns.get(i)
+                if col in ("i_evalue", "pvalue", "bgc_probability"):
+                    getattr(table, col).append(float(value))
+                elif col in ("start", "end", "domain_start", "domain_end"):
+                    getattr(table, col).append(int(value))
+                elif col in cls.__annotations__:
+                    getattr(table, col).append(value)
 
         return table
 
@@ -692,6 +722,13 @@ class ClusterTable(Dumpable, Sized):
             table.domains.append(sorted(domains))
         return table
 
+    def __iadd__(self, rhs: "ClusterTable") -> "ClusterTable":
+        if not isinstance(rhs, FeatureTable):
+            return NotImplemented
+        for col in self.__annotations__:
+            getattr(self, col).extend(getattr(rhs, col))
+        return self
+
     def __len__(self) -> int:  # noqa: D105
         return len(self.sequence_id)
 
@@ -757,33 +794,49 @@ class ClusterTable(Dumpable, Sized):
         reader = csv.reader(fh, dialect=dialect)
         header = next(reader)
 
-        # find the columns and the type of each column based on the annotations
-        columns = [
-            (
-                (header.index(col) if col in header else None),
-                col,
-                getattr(table, col).append,
-                getattr(ty.__args__[0], "__args__", ty.__args__)[0], # concrete type
-            )
-            for col, ty in cls.__annotations__.items()
-        ]
+        # get the name of each column
+        columns = {i:col for i, col in enumerate(header)}
 
-        # check that no column is missing
-        missing = [c for i, c, _, _ in columns if i is None]
-        if missing:
-            raise ValueError("table is missing columns: {}".format(", ".join(missing)))
+        # check that if a column is missing, it is one of the optional values
+        missing = set(cls.__annotations__).difference(columns.values())
+        missing_required = missing.difference({
+            "average_p",
+            "max_p",
+            "type",
+            "alkaloid_probability",
+            "polyketide_probability",
+            "ripp_probability",
+            "saccharide_probability",
+            "terpene_probability",
+            "nrp_probability",
+            "other_probability",
+            "proteins",
+            "domains"
+        })
+        if missing_required:
+            raise ValueError("table is missing columns: {}".format(", ".join(missing_required)))
 
         # extract elements from the CSV rows
-        for line in reader:
-            for index, column, append, ty in columns:
-                if ty is ProductType:
-                    types = [ ProductType.__members__[k] for k in line[index].split(";") ]
-                    append(functools.reduce(operator.or_, types))
-                elif column == "proteins" or column == "domains":
-                    append(list(line[index].split(";")))
-                elif isinstance(ty, type):
-                    append(ty(line[index]))
+        for row in reader:
+            for col in missing:
+                if col in ("proteins", "domains"):
+                    getattr(table, col).append(list())
+                elif col in ("average_p", "max_p"):
+                    getattr(table, col).append(1.0)
+                elif col == "type":
+                    table.type.append(ProductType.Unknown)
                 else:
-                    raise ValueError
+                    getattr(table, col).append(0.0)
+            for i,value in enumerate(row):
+                col = columns.get(i)
+                if col in ("i_evalue", "pvalue") or col.endswith(("_p", "_probability")):
+                    getattr(table, col).append(float(value))
+                elif col in ("start", "end", "domain_start", "domain_end"):
+                    getattr(table, col).append(int(value))
+                elif col == "type":
+                    types = [ProductType.__members__[x] for x in value.split(";")]
+                    table.type.append(ProductType.pack(types))
+                elif col in cls.__annotations__:
+                    getattr(table, col).append(value)
 
         return table
